/*
 *
 * Configuration.h
 *
 * Copyright (c) 2002 INFN, CERN on behalf of the EU DataGrid.
 * For license conditions see LICENSE file or 
 * http://www.edg.org/license.html
 * 
 */

#ifndef EDGReplicaManager_Configuration_H
#define EDGReplicaManager_Configuration_H

/*
 * @author <a href="mailto:Giuseppe.Andronico@ct.infn.it">Heinz Stockinger</a>
 * @author <a href="mailto:Flavia.Donno@pi.infn.it">Flavia Donno</a>
 * @author <a href="mailto:Heinz.Stockinger@cern.ch">Giuseppe Andronico</a>
 * @author <a href="mailto:Peter.Kunszt@cern.ch">Peter Kunszt</a>
 * @version $Id: Configuration.h,v 1.6 2004/06/14 12:01:17 jamesc Exp $
 */

#include <string>
#include <map>
#include <fstream>

#include "EdgReplicaManager/ReplicaManagerExceptions.h"

namespace EdgReplicaManager {

    class Configuration {
    public:


	/** A user defined config file is used.
	 */ 
	Configuration(const std::string &configFile) throw(ReplicaManagerException);

	/**Default Destructor. 
	 */
	~Configuration(void);

	/** returns a single value of the config file.
	 */
	std::string getProperty (const std::string &key) throw(ReplicaManagerException);

	/** returns exit status of the constructor invocation
	 */

    private:

	/** reads the config file on demand. Note that this public method is also 
	    called by the constructor and does not need to be used explicitly when
	    the object is instantiated.
	*/
	void readConfFile(void)  throw(ReplicaManagerException);

	/** trim a string from left and right
	 */
	void trim(std::string & input);

	std::ifstream m_fconf;
	std::string m_configFile;
	std::map<std::string, std::string> m_envNames;

    };

};

/*! \class ConfigHandler
 *  \brief handle configuration parameters
 *  \param fconf_               handler to the configuration file
 *  \param ConfigFile_          full name of gdmp configuration file
 *  \param envNames_		map to hold config key-value pair
 * this class is intended so that can be instantiated only one time.
 * if a string is given with a filename it read the file for configurations
 */



#endif
