/*
 *
 * InfoService.h
 *
 * Copyright (c) 2002 INFN, CERN on behalf of the EU DataGrid.
 * For license conditions see LICENSE file or 
 * http://www.edg.org/license.html
 * 
 */

#ifndef EDGReplicaManager_InfoService_H
#define EDGReplicaManager_InfoService_H 1

/*
 * @author <a href="mailto:Peter.Kunszt@cern.ch">Peter Kunszt</a>
 * @version $Id: InfoService.h,v 1.7 2004/06/14 12:01:17 jamesc Exp $
 */

#include <string>
#include <vector>

#include "EdgReplicaManager/ReplicaManagerExceptions.h"

namespace EdgReplicaManager {

    //! A simple interface to the EDG Replica Manager
    class InfoService {
    public:

	/** Constructor
	 */

	InfoService(const std::string &vo, bool secure);

	/** Standard virtual destructor
	 */
	virtual ~InfoService() {}

	/**
	 * return the single LRC instance for the system
	 *
	 */
	std::string getLocalReplicaCatalogLocation(const std::string &domain) throw(InfoServiceException);

	/**
	 * return the single RMC instance for the system
	 *
	 */
	std::string getMetadataCatalogLocation(const std::string &domain) throw(InfoServiceException);

	/**
	 * return the single ROS instance for the system
	 */
	std::string getReplicaOptimizationLocation(const std::string &domain) throw(InfoServiceException);

	/** 
	 * get service
	 */
	std::string getServiceURI(const std::string &serviceType, const std::string &domain) 
	    throw(InfoServiceException);

	//! get close SE based on a CE id
	virtual std::string getCloseSE(const std::string& ceID) throw ( InfoServiceException ) = 0;

	bool isSecure() {
	    return m_secure;
	}
    protected:

	virtual const std::vector<std::string>& getServiceURIs(const std::string &serviceType, 
							       const std::string &domain) 
	    throw(InfoServiceException) =0;


	bool isLRC(const std::string &type);
	bool isRMC(const std::string &type);
	bool isROS(const std::string &type);

	std::string m_lrcURI;
	std::string m_rmcURI;
	std::string m_rosURI;
	std::string m_vo;
	bool m_secure;
    };

};

#endif
