/*
 *
 * InfoServiceMDS.h
 *
 * Copyright (c) 2002 INFN, CERN on behalf of the EU DataGrid.
 * For license conditions see LICENSE file or 
 * http://www.edg.org/license.html
 * 
 */

#ifndef EDGReplicaManager_InfoServiceMDS_H
#define EDGReplicaManager_InfoServiceMDS_H 1

/*
 * @author <a href="mailto:Peter.Kunszt@cern.ch">Peter Kunszt</a>
 */

#include "EdgReplicaManager/InfoService.h"
#include "EdgReplicaManager/LDAPQuery.h"

namespace EdgReplicaManager {

    //! Info service based on a file
    class InfoServiceMDS : public InfoService {
    public:

	/** Constructor
	 */

	InfoServiceMDS(const std::string &vo, bool secure, const std::string &url, const std::string &root )  
	    throw ( InfoServiceException );

	/** Destructor
	 */
        ~InfoServiceMDS();

	//! get service
        const std::vector<std::string>& getServiceURIs(const std::string &serviceType, 
				  const std::string &domain) 
	    throw(InfoServiceException);

	//! get close SE based on a CE id
	std::string getCloseSE(const std::string& ceID) throw ( InfoServiceException );

    private:
	void initLDAP() throw(LDAPException);
	const std::vector<std::string>& doLDAPQuery(const std::string& constraint,
						    const std::string& attr) throw(LDAPException);

	std::string m_root;
	std::string m_url;
	LDAPQuery* m_ldap;

    };

};

#endif
