/*
 *
 * LDAPQuery.h
 *
 * Copyright (c) 2002 INFN, CERN on behalf of the EU DataGrid.
 * For license conditions see LICENSE file or 
 * http://www.edg.org/license.html
 * 
 */

#ifndef EDGReplicaManager_LDAPQuery_H
#define EDGReplicaManager_LDAPQuery_H

/*
 * @author <a href="mailto:Heinz.Stockinger@cern.ch">Giuseppe Andronico</a>
 * @author <a href="mailto:Giuseppe.Andronico@ct.infn.it">Heinz Stockinger</a>
 * @author <a href="mailto:Peter.Kunszt@cern.ch">Peter Kunszt</a>
 * @version $Id: LDAPQuery.h,v 1.3 2004/06/14 12:01:17 jamesc Exp $
 */

#include <sys/time.h>

#include <string>
#include <vector>
#include <map>
#include <list>

//LDAP includes
#include "lber.h"
#include "ldap.h"

#include "EdgReplicaManager/ReplicaManagerExceptions.h"

namespace EdgReplicaManager {

    /**class to query an LDAP server
     */

    class LDAPQuery {
    public: 
	LDAPQuery(const std::string& baseDN, const std::string& hostname) throw(LDAPException);

	~LDAPQuery();

	/** do a query on LDAP server and set to the first entry */
	const std::vector<std::string>& query(const std::string& constraint,
					      const std::string& attr) throw(LDAPException);


    private:

	/** base distinguished name , e.g.: "o=Grid" */
	const std::string m_baseDN;
	/** name of the host we want query, e.g.: testbed008.cern.ch */
	std::string m_hostname;
	/** TCP/IP port, e.g:2170 */
	int m_port;
	/** LDAP handle */
	LDAP* m_handle;
	/** LDAP timeout */
	struct timeval m_timeout;
	/** pointer to a LDAP structure */
	std::vector<std::string> m_buffer;
	/** contain the result of query */
	LDAPMessage* m_result;

    };

};

#endif
