/*
 *
 * ReplicaManager.h
 *
 * Copyright (c) 2002 INFN, CERN on behalf of the EU DataGrid.
 * For license conditions see LICENSE file or 
 * http://www.edg.org/license.html
 * 
 */

#ifndef ReplicaManager_H
#define ReplicaManager_H 1

/*
 * @author <a href="mailto:Andrea.Domenici@iet.unipi.it">Andrea Domenici</a>
 */

#include <string>
#include <vector>

#include "EdgReplicaOptimization/ReplicaOptimization.h"
#include <EdgReplicaLocationService/ReplicationExceptions.h>
#include <EdgReplicaMetadataCatalog/ReplicationExceptions.h>
#include <EdgReplicaManager/ReplicaManagerExceptions.h>

/**
 * Replica Manager namespace. All classes of the replica manager are part of this namespace.
 */

namespace EdgReplicaManager {

    //! A simple interface to the EDG Replica Manager
    class ReplicaManager {
    public:
	typedef std::vector<std::string>            	LFNList_t;
	typedef std::vector<std::string>            	SFNList_t;
	typedef std::vector<std::string>            	LRCList_t;
	typedef std::vector<std::string>            	CEList_t;
	typedef std::vector<std::string>            	SEList_t;
	typedef std::vector<std::string>            	Protocols_t;
	typedef std::vector<EdgReplicaOptimization::AccessCost> AccessCosts_t;

	/** Standard virtual destructor
	 */
	virtual ~ReplicaManager() {};

	//! get list of replicas
	virtual void listReplicas(std::string fileID, SFNList_t & sfns)
	    throw ( ReplicaManagerException, 
		    EdgReplicaLocationService::NoSuchGuidException,
		    EdgReplicaLocationService::NoSuchPfnException,
		    EdgReplicaMetadataCatalog::NoSuchAliasException,
		    EdgReplicaMetadataCatalog::CommunicationException,
		    EdgReplicaLocationService::CommunicationException ) = 0;

	//! get access cost
	virtual void getAccessCost(LFNList_t LFNList,
				   CEList_t CEList,
				   //  SEList_t CloseSEList,
				   Protocols_t protocols,
				   AccessCosts_t &accessCost)
	    throw ( ReplicaManagerException,
		    EdgReplicaOptimization::ReplicationException,
		    EdgReplicaLocationService::NoSuchGuidException,
		    EdgReplicaLocationService::NoSuchPfnException ,
		    EdgReplicaMetadataCatalog::NoSuchAliasException,
		    EdgReplicaMetadataCatalog::CommunicationException,
		    EdgReplicaLocationService::CommunicationException) = 0;

	//EdgReplicaOptimization::ReplicaOptimization::AccessCosts_t
	//getAccessCost(LFNList_t   LFNList,
        //              CEList_t    CEList,
        //              std::string protocol);
	virtual std::string listBestFile(std::string file, std::string seHost)
	    throw ( ReplicaManagerException,
		    EdgReplicaOptimization::ReplicationException,
		    EdgReplicaLocationService::NoSuchGuidException,
		    EdgReplicaLocationService::NoSuchPfnException ,
		    EdgReplicaMetadataCatalog::NoSuchAliasException,
		    EdgReplicaMetadataCatalog::CommunicationException,
		    EdgReplicaLocationService::CommunicationException ) = 0;

	//! set the secure flag. all services will be tried to be contacted in secure mode.
	virtual void setSecure() = 0;
    };

};

#endif
