/* -*- C++ -*-
 *
 * ReplicaManagerExceptions.h
 *
 * Copyright (c) 2002 CERN on behalf of the EU DataGrid.
 * For license conditions see LICENSE file or 
 * http://www.edg.org/license.html
 * 
 */

#include <exception>
#include <string>

#ifndef ReplicaManagerExceptions_H
#define ReplicaManagerExceptions_H 1

/*
 *
 * @author <a href="mailto:james.casey@cern.ch">James Casey</a>
 * @author <a href="mailto:Radovan.Chytracek@cern.ch">Radovan Chytracek</a>
 * @author <a href="mailto:Peter.Kunszt@cern.ch">Peter Kunszt</a>
 * @version $Id: ReplicaManagerExceptions.h,v 1.7 2004/06/14 12:01:17 jamesc Exp $
 */
namespace EdgReplicaManager {

  /**
   * A simple top-level exception class for ReplicaManager Exceptions.
   *
   */
  class ReplicaManagerException : public std::exception {
    public:

    public:
      ReplicaManagerException() throw() { }

      ReplicaManagerException(const std::string& what) throw() 
       : m_what ("Replica Manager C++ API: "+what) { }

      virtual ~ReplicaManagerException() throw(){ }

      virtual const char* what () const throw() {
        return m_what.c_str ();
      }

    private:
      std::string m_what;
  };

  /**
   * An exception thrown on info service problems
   */
  class InfoServiceException : public ReplicaManagerException {
    public: 
      InfoServiceException(const std::string& msg) 
	  : ReplicaManagerException("InfoService: " + msg) { }

      virtual ~InfoServiceException() throw(){ }

  };

  /**
   * An exception thrown on LDAP problems
   */
  class LDAPException : public InfoServiceException {
    public: 
      LDAPException(const std::string& msg) 
	  : InfoServiceException("LDAP-" + msg) { }

      virtual ~LDAPException() throw(){ }

  };

  /**
   * An exception thrown on URI problems
   */
  class URIException : public ReplicaManagerException {
    public: 
      URIException(const std::string& msg) 
	  : ReplicaManagerException(msg) { }

      virtual ~URIException() throw(){ }

  };

  /**
   * An exception thrown when we don't implement a method yet
   */
  class NotImplementedException : public ReplicaManagerException {
    public: 
      NotImplementedException(const std::string& method) 
	  : ReplicaManagerException("Not Implemented: " + method),
	    m_method(method) { }

      virtual ~NotImplementedException() throw(){ }

      const std::string& getMethod() const {
        return m_method;
      }
      
    private:
      const std::string m_method;
  };

}


#endif // ReplicaManagerExceptions_H

