/*
 *
 * ReplicaManagerImpl.h
 *
 * Copyright (c) 2002 INFN, CERN on behalf of the EU DataGrid.
 * For license conditions see LICENSE file or 
 * http://www.edg.org/license.html
 * 
 */

#ifndef ReplicaManagerImpl_H
#define ReplicaManagerImpl_H 1

/*
 * @author <a href="mailto:Peter.Kunszt@cern.ch">Peter Kunszt</a>
 * @author <a href="mailto:Andrea.Domenici@iet.unipi.it">Andrea Domenici</a>
 */

#include <string>
#include <vector>

#include "EdgReplicaLocationService/LocalReplicaCatalog.h"
#include "EdgReplicaMetadataCatalog/ReplicaMetadataCatalog.h"
#include "EdgReplicaManager/ReplicaManager.h"
#include "EdgReplicaManager/InfoService.h"
#include "EdgReplicaManager/Configuration.h"

namespace EdgReplicaManager {

    //! implementation
    class ReplicaManagerImpl : public ReplicaManager {
    public:

	/** 
	 * create a new ReplicaOptimization object
	 */

	ReplicaManagerImpl(const std::string& vo)  throw( ReplicaManagerException);

	ReplicaManagerImpl(const std::string& vo, Configuration *conf)  throw( ReplicaManagerException);

	~ReplicaManagerImpl();

	//! get list of replicas
	void listReplicas(std::string fileID, SFNList_t & sfns) 
	    throw ( ReplicaManagerException, 
		    EdgReplicaLocationService::NoSuchGuidException,
		    EdgReplicaLocationService::NoSuchPfnException,
		    EdgReplicaMetadataCatalog::NoSuchAliasException,
		    EdgReplicaMetadataCatalog::CommunicationException,
		    EdgReplicaLocationService::CommunicationException );

	//! get access cost
	void getAccessCost(LFNList_t LFNList,
			   CEList_t CEList,
			   Protocols_t protocols,
			   AccessCosts_t &accessCost) 
	    throw ( ReplicaManagerException,
		    EdgReplicaOptimization::ReplicationException,
		    EdgReplicaLocationService::NoSuchGuidException,
		    EdgReplicaLocationService::NoSuchPfnException ,
		    EdgReplicaMetadataCatalog::NoSuchAliasException,
		    EdgReplicaMetadataCatalog::CommunicationException,
		    EdgReplicaLocationService::CommunicationException);

	std::string listBestFile(std::string file, std::string seHost) 
	    throw ( ReplicaManagerException,
		    EdgReplicaOptimization::ReplicationException,
		    EdgReplicaLocationService::NoSuchGuidException,
		    EdgReplicaLocationService::NoSuchPfnException ,
		    EdgReplicaMetadataCatalog::NoSuchAliasException,
		    EdgReplicaMetadataCatalog::CommunicationException,
		    EdgReplicaLocationService::CommunicationException );

	void setSecure() {
	    m_secure = true;
	}

    private:

	/* private methods */

	void init()  throw( ReplicaManagerException);
	void initInfoService(const std::string& vo)  throw( ReplicaManagerException);

        void initLRC() throw(InfoServiceException);
        void initROS() throw(InfoServiceException);
        void initRMC() throw(InfoServiceException);

	bool startsWith(const std::string, const std::string);
	bool isLFN(std::string file);
	bool isGUID(std::string file);
	bool isSURL(std::string file);

	float convert2float( const char* const numstr );


  std::string withGuidPrefix(std::string str);
  std::string stripGuidPrefix(std::string str);
  std::string withLfnPrefix(std::string str);
  std::string stripLfnPrefix(std::string str);

	/* private member variables */

	// the service endpoints
	std::string m_lrcURL;
	std::string m_rosURL;
	std::string m_rmcURL;
	std::string m_localdomain;

	// the service clients
	EdgReplicaLocationService::LocalReplicaCatalog* m_lrc;
	EdgReplicaMetadataCatalog::ReplicaMetadataCatalog* m_rmc;
	EdgReplicaOptimization::ReplicaOptimization* m_ros;

	// info and configuration
	InfoService *m_infoService;
	Configuration *m_config;
	const std::string m_vo;

	// helpers
	bool m_myconfig;
	bool m_fakeROS;
	bool m_fullRLS;
	bool m_secure;
	std::string m_edgloc;

    };
};

#endif
